/*
 * U-boot - bf1.h  Configuration file for BlackfinOne board
 *			having BF532 processor
 *
 * Copyright (c) 2006 Intratrade Ltd., Ivan Danov, idanov@gmail.com
 *
 * Copyright (c) 2005 blackfin.uclinux.org
 *
 * (C) Copyright 2000-2004
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_BF1_H__
#define __CONFIG_BF1_H__

/*
 * Board settings
 *
 */

#define __ADSPLPBLACKFIN__		1
#define __ADSPBF532__			1
#define __ADSPBF533__			1
#define CONFIG_STAMP			1
#define CONFIG_BF1			1
#define CONFIG_RTC_BF533		1
#define CONFIG_SPI			1

// To execute u-boot from SPI flash
//#define EXEC_FROM_SPI			1

// See cmd_eeprom.c
#define CONFIG_SPI_PAGE_WRITE_SIZE	0x800000
// for M25P64 - display properties only
#define CONFIG_SPI_PAGE_READ_SIZE	0x1000

/* FLASH/ETHERNET uses the same address range */
#define SHARED_RESOURCES		1

#define CONFIG_VDSP			0

/*
 * Clock settings
 *
 */

/* CONFIG_CLKIN_HZ is any value in Hz				 */
#define CONFIG_CLKIN_HZ			10000000
/* CONFIG_CLKIN_HALF controls what is passed to PLL 0=CLKIN	 */
/*						    1=CLKIN/2	 */
#define CONFIG_CLKIN_HALF		0
/* CONFIG_PLL_BYPASS controls if the PLL is used 0=don't bypass	 */
/*						 1=bypass PLL	 */
#define CONFIG_PLL_BYPASS		0
/* CONFIG_VCO_MULT controls what the multiplier of the PLL is.	 */
/* Values can range from 1-64					 */
#define CONFIG_VCO_MULT			40
/* CONFIG_CCLK_DIV controls what the core clock divider is	 */
/* Values can be 1, 2, 4, or 8 ONLY				 */
#define CONFIG_CCLK_DIV			1
/* CONFIG_SCLK_DIV controls what the peripheral clock divider is */
/* Values can range from 1-15					 */
#define CONFIG_SCLK_DIV			3
/* CONFIG_SPI_BAUD controls the SPI peripheral clock divider     */
/* Values can range from 2-65535                                 */
/* SCK Frequency = SCLK / (2 * CONFIG_SPI_BAUD)                  */
#define CONFIG_SPI_BAUD			2

/*
 * Network Settings
 */
/* network support */
#define CONFIG_IPADDR		192.168.1.30
#define CONFIG_NETMASK		255.255.255.0
#define CONFIG_GATEWAYIP	192.168.1.1
#define CONFIG_SERVERIP		192.168.1.2
#define CONFIG_HOSTNAME		BF1
#define CONFIG_ROOTPATH		/bf1/rootfs

#define CONFIG_ENV_OVERWRITE	1
#define CONFIG_HAS_ETH1		1

/* To remove hardcoding and enable MAC storage in EEPROM  */
#define CONFIG_ETHADDR		02:80:ad:20:31:b8
#define CONFIG_ETH1ADDR		02:80:ad:20:31:b9
#define CONFIG_NET_MULTI	1

/*
 * SDRAM settings
 *
 */

#define CONFIG_MEM_SIZE			64		/* 128, 64, 32, 16 */

#define CFG_MEMTEST_START		0x00000000	/* memtest works on */
#define CFG_SDRAM_BASE			0x00000000
#define CFG_LOAD_ADDR           	0x01000000      /* default load address */

#if (CONFIG_MEM_SIZE == 64 )
	#define CONFIG_MEM_ADD_WDTH		10	       /* 8, 9, 10, 11	  */
	#define CONFIG_MEM_MT48LC32M16A2TG_75	1
	#define CFG_MAX_RAM_SIZE       	 	0x04000000
	#define CFG_MEMTEST_END         	0x03EFFFFF      /* 1 ... 63 MB in DRAM */
#endif
#if (CONFIG_MEM_SIZE == 32 )
	#define CONFIG_MEM_ADD_WDTH		9	       /* 8, 9, 10, 11	  */
	#define CONFIG_MEM_MT48LC16M16A2TG_7E	1
	#define CFG_MAX_RAM_SIZE        	0x02000000
	#define CFG_MEMTEST_END         	0x01EFFFFF      /* 1 ... 31 MB in DRAM */
#endif


/*
 * Command settings
 *
 */

#define CFG_LONGHELP			1

#define CONFIG_BOOTDELAY		3

/* Enable this if bootretry required, currently its disabled */
#define CONFIG_BOOT_RETRY_TIME		-1

#ifdef EXEC_FROM_SPI
#define CONFIG_BOOTCOMMAND		"run spiboot"
#else
#define CONFIG_BOOTCOMMAND		"run flashboot"
#endif

#define CONFIG_AUTOBOOT_PROMPT		"autoboot in %d seconds\n"

#define CONFIG_COMMANDS			(CONFIG_CMD_DFL | \
					 CFG_CMD_PING	| \
					 CFG_CMD_DHCP	| \
					 CFG_CMD_SNTP	| \
					 CFG_CMD_EEPROM	| \
					 CFG_CMD_ELF	| \
					 CFG_CMD_CACHE	| \
					 CFG_CMD_JFFS2	| \
					 CFG_CMD_DATE)

#ifdef EXEC_FROM_SPI
#define CONFIG_BOOTARGS "root=/dev/mtdblock5 rootfstype=jffs2 rw"
#else
#define CONFIG_BOOTARGS "root=/dev/mtdblock2 rootfstype=jffs2 rw"
#endif

#define STR2(x) #x
#define STR(x) STR2(x)
#define CONFIG_EXTRA_ENV_SETTINGS1					\
	"addnet=setenv bootargs $(bootargs) "				\
		"ip=$(ipaddr):$(serverip):$(gatewayip):"		\
		"$(netmask):$(hostname):eth0:off\0"			\
	"nfsargs=setenv bootargs root=/dev/nfs rw "			\
		"nfsroot=$(serverip):$(rootpath);run addnet;\0"		\
	"ramboot=tftp "STR(CFG_LOAD_ADDR)" vmImage;run addnet;bootm\0"	\
	"nfsboot=tftp "STR(CFG_LOAD_ADDR)" vmImage;run nfsargs;bootm\0"	\
	"flashboot=run addnet;bootm 20020000\0"				\
	"spiboot=eeprom read " STR(CFG_LOAD_ADDR) " 30000 110000;" 	\
		"run addnet;bootm " STR(CFG_LOAD_ADDR) "\0"		\
        "update=tftpboot " STR(CFG_LOAD_ADDR) " u-boot.bin;"		\
                "protect off 0x20000000 0x2001FFFF;"			\
		"erase 0x20000000 0x2001FFFF;"				\
                "cp.b " STR(CFG_LOAD_ADDR) " 0x20000000 $(filesize)\0"	\
	"\0"

#ifdef CONFIG_NET_MULTI

#define CONFIG_EXTRA_ENV_SETTINGS				\
	CONFIG_EXTRA_ENV_SETTINGS1				\
	"ethprime=DM9000#0\0"					\
	"ethact=DM9000#0\0"

#else

#define CONFIG_EXTRA_ENV_SETTINGS				\
	CONFIG_EXTRA_ENV_SETTINGS1

#endif

/* This must be included AFTER the definition of CONFIG_COMMANDS (if any) */
#include <cmd_confdefs.h>

/*
 * Console settings
 *
 */

#define CONFIG_BAUDRATE			115200
#define CFG_BAUDRATE_TABLE		{ 9600, 19200, 38400, 57600, 115200 }

#ifdef EXEC_FROM_SPI
#define CFG_PROMPT			"bf1s>"	/* Monitor Command Prompt */
#else
#define CFG_PROMPT			"bf1>"	/* Monitor Command Prompt */
#endif
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CFG_CBSIZE			1024	/* Console I/O Buffer Size */
#else
#define CFG_CBSIZE			256	/* Console I/O Buffer Size */
#endif
#define CFG_PBSIZE			(CFG_CBSIZE+sizeof(CFG_PROMPT)+16)	/* Print Buffer Size */
#define CFG_MAXARGS			16	/* max number of command args */
#define CFG_BARGSIZE			CFG_CBSIZE	/* Boot Argument Buffer Size */

#define CONFIG_LOADS_ECHO		1

/*
 * Network settings
 *
 */
#define CONFIG_DRIVER_ETHER             1
#define CONFIG_DRIVER_DM9000            1
#define CONFIG_DRIVER_DM9000A           1
#undef CONFIG_DM9000_HAVE_EEPROM       
#define CONFIG_NET_RETRY_COUNT          50
#define CONFIG_TFTP_TIMEOUT             250
#define CONFIG_DM9000_BASE              0x20000010
#define DM9000_IO                       CONFIG_DM9000_BASE
#define DM9000_DATA                     (CONFIG_DM9000_BASE+2)
#define CONFIG_DM9000_2_BASE            0x20000020
#define DM9000_2_IO                     CONFIG_DM9000_2_BASE
#define DM9000_2_DATA                   (CONFIG_DM9000_2_BASE+2)
#undef  CONFIG_DM9000_USE_8BIT
#define CONFIG_DM9000_USE_16BIT         1
#undef  CONFIG_DM9000_USE_32BIT
#undef  CONFIG_DM9000_DEBUG
//#define CONFIG_DM9000_DEBUG           1

/* To remove hardcoding and enable MAC storage in EEPROM */
/* #define HARDCODE_MAC			1 */

/*
 * Flash settings
 *
 */

#define CFG_FLASH_CFI				/* The flash is CFI compatible	*/
#define CFG_FLASH_CFI_DRIVER			/* Use common CFI driver	*/
#define CFG_FLASH_CFI_AMD_RESET


#define CFG_FLASH_BASE			0x20000000
#define CFG_MAX_FLASH_BANKS		1		/* max number of memory banks */
#define CFG_MAX_FLASH_SECT		71		/* max number of sectors on one chip */

#ifdef EXEC_FROM_SPI

#define CFG_ENV_IS_IN_EEPROM		1
#define CONFIG_PREBOOT			"echo starting from spi flash"
#define CFG_ENV_OFFSET                  0x20000
#define CFG_ENV_HEADER                  (CFG_ENV_OFFSET + 0x12A)        /* 0x12A is the length of LDR file header */
#define CFG_ENV_SIZE                    0x10000
#define CFG_ENV_SECT_SIZE               0x10000 /* Total Size of Environment Sector */

#else

#define CFG_ENV_IS_IN_FLASH		1
#define CONFIG_PREBOOT			"echo starting from nor flash"
#define CFG_ENV_ADDR			0x203fe000
#define CFG_ENV_SIZE			0x2000
#define CFG_ENV_SECT_SIZE		0x2000 /* Total Size of Environment Sector */

#endif

#define CFG_FLASH_ERASE_TOUT		30000	/* Timeout for Chip Erase (in ms) */
#define CFG_FLASH_ERASEBLOCK_TOUT	5000	/* Timeout for Block Erase (in ms) */
#define CFG_FLASH_WRITE_TOUT		1	/* Timeout for Flash Write (in ms) */

/* JFFS Partition offset set  */
#define CFG_JFFS2_FIRST_BANK 0
#define CFG_JFFS2_NUM_BANKS  1
/* 512k reserved for u-boot */
#define CFG_JFFS2_FIRST_SECTOR		11

/*
 * following timeouts shall be used once the
 * Flash real protection is enabled
 */
#define CFG_FLASH_LOCK_TOUT		5	/* Timeout for Flash Set Lock Bit (in ms) */
#define CFG_FLASH_UNLOCK_TOUT		10000	/* Timeout for Flash Clear Lock Bits (in ms) */

/*
 * I2C settings
 * By default PF2 is used as SDA and PF3 as SCL on the Stamp board
 */
#undef CONFIG_SOFT_I2C
/*
 * Software (bit-bang) I2C driver configuration
 */
#define PF_SCL				PF3
#define PF_SDA				PF2

#define I2C_INIT			(*pFIO_DIR |=  PF_SCL); asm("ssync;")
#define I2C_ACTIVE			(*pFIO_DIR |=  PF_SDA); *pFIO_INEN &= ~PF_SDA; asm("ssync;")
#define I2C_TRISTATE			(*pFIO_DIR &= ~PF_SDA); *pFIO_INEN |= PF_SDA; asm("ssync;")
#define I2C_READ			((volatile)(*pFIO_FLAG_D & PF_SDA) != 0); asm("ssync;")
#define I2C_SDA(bit)			if(bit) { \
							*pFIO_FLAG_S = PF_SDA; \
							asm("ssync;"); \
						} \
					else	{ \
							*pFIO_FLAG_C = PF_SDA; \
							asm("ssync;"); \
						}
#define I2C_SCL(bit)			if(bit) { \
							*pFIO_FLAG_S = PF_SCL; \
							asm("ssync;"); \
						} \
					else	{ \
							*pFIO_FLAG_C = PF_SCL; \
							asm("ssync;"); \
						}
#define I2C_DELAY			udelay(5)	/* 1/4 I2C clock duration */

#define CFG_I2C_SPEED			50000
#define CFG_I2C_SLAVE			0xFE

/*
 * Compact Flash settings
 */

/* Enabled below option for CF support */
/* #define CONFIG_STAMP_CF		1 */

#if defined(CONFIG_STAMP_CF) && (CONFIG_COMMANDS & CFG_CMD_IDE)

#define CONFIG_MISC_INIT_R		1
#define CONFIG_DOS_PARTITION		1

/*
 * IDE/ATA stuff
 */
#undef	CONFIG_IDE_8xx_DIRECT		/* no pcmcia interface required */
#undef	CONFIG_IDE_LED			/* no led for ide supported */
#undef	CONFIG_IDE_RESET		/* no reset for ide supported */

#define CFG_IDE_MAXBUS	1		/* max. 1 IDE busses */
#define CFG_IDE_MAXDEVICE		(CFG_IDE_MAXBUS*1) /* max. 1 drives per IDE bus */

#define CFG_ATA_BASE_ADDR		0x20200000
#define CFG_ATA_IDE0_OFFSET		0x0000

#define CFG_ATA_DATA_OFFSET		0x0020	/* Offset for data I/O */
#define CFG_ATA_REG_OFFSET		0x0020	/* Offset for normal register accesses */
#define CFG_ATA_ALT_OFFSET		0x0007	/* Offset for alternate registers */

#define CFG_ATA_STRIDE			2
#endif

#define CFG_MONITOR_LEN			(128 << 10)	/* Reserve 128 kB for Monitor	*/
#define CFG_MONITOR_BASE		(CFG_MAX_RAM_SIZE - CFG_MONITOR_LEN)

#if ( CONFIG_CLKIN_HALF == 0 )
#define CONFIG_VCO_HZ			( CONFIG_CLKIN_HZ * CONFIG_VCO_MULT )
#else
#define CONFIG_VCO_HZ			(( CONFIG_CLKIN_HZ * CONFIG_VCO_MULT ) / 2 )
#endif

#if (CONFIG_PLL_BYPASS == 0)
#define CONFIG_CCLK_HZ			( CONFIG_VCO_HZ / CONFIG_CCLK_DIV )
#define CONFIG_SCLK_HZ			( CONFIG_VCO_HZ / CONFIG_SCLK_DIV )
#else
#define CONFIG_CCLK_HZ			CONFIG_CLKIN_HZ
#define CONFIG_SCLK_HZ			CONFIG_CLKIN_HZ
#endif

/*
 * Miscellaneous configurable options
 */
#define CFG_HZ				1000		/* 1ms time tick */

#define CFG_MALLOC_LEN			(128 << 10)	/* Reserve 128 kB for malloc()	*/
#define CFG_MALLOC_BASE			(CFG_MONITOR_BASE - CFG_MALLOC_LEN)
#define CFG_GBL_DATA_SIZE		0x4000
#define CFG_GBL_DATA_ADDR		(CFG_MALLOC_BASE - CFG_GBL_DATA_SIZE)
#define CONFIG_STACKBASE		(CFG_GBL_DATA_ADDR  - 4)

#define CFG_LARGE_IMAGE_LEN	0x4000000	/* Large Image Length, set to 64 Meg */

#undef CONFIG_SHOW_BOOT_PROGRESS

/*
 * Stack sizes
 */
#define CONFIG_STACKSIZE		(128*1024)	/* regular stack */

/*
 * FLASH organization and environment definitions
 */
#define CFG_BOOTMAPSZ			(8 << 20)	/* Initial Memory map for Linux */

/* 0xFF, 0xBBC3BBc3, 0x99B39983 */
/*#define AMGCTLVAL		(AMBEN_P0 | AMBEN_P1 | AMBEN_P2 | AMCKEN)
#define AMBCTL0VAL		(B1WAT_11 | B1RAT_11 | B1HT_3 | B1ST_4 | B1TT_4 | B1RDYPOL |	\
				B1RDYEN | B0WAT_11 | B0RAT_11 | B0HT_3 | B0ST_4 | B0TT_4 | B0RDYPOL | B0RDYEN)
#define AMBCTL1VAL		(B3WAT_9 | B3RAT_9 | B3HT_2 | B3ST_3 | B3TT_4 | B3RDYPOL |	\
				B3RDYEN | B2WAT_9 | B2RAT_9 | B2HT_2 | B2ST_4 | B2TT_4 | B2RDYPOL | B2RDYEN)
*/
#define AMGCTLVAL		0xFF
#define AMBCTL0VAL		0xffc2ffc2
#define AMBCTL1VAL		0xffc2ffc2
#define CF_AMBCTL1VAL		0xffc2ffc2

#ifdef CONFIG_VDSP
#define ET_EXEC_VDSP		0x8
#define SHT_STRTAB_VDSP		0x1
#define ELFSHDRSIZE_VDSP	0x2C
#define VDSP_ENTRY_ADDR		0xFFA00000
#endif

#endif
